////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// RANK_q4.mod
//
// August 2021
//
// Replication codes: "Monetary Policy with Capital Misallocation"
//
// (c) Beatriz Gonzalez, Galo Nuno, Dominik Thaler and Silvia Albrizio
//
// This is the main file to solve for the optimal monetary policy in complete markets economy
//
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// prepare model definition
var w r K C L i Y m pi T Q Welf rho_hh_t Aux_Va q R iota mbar G epsilon_t; 


parameters rho rho_hh delta gamma alpha sigma theta Deltat  Deltaz  psi Upsilon  epsilon phiTR pibar thetaP eta_q share_d rhoTR phiK N_period Zhat wedge;

  
model_parameters
upsilon_RANK

rho      = params(1);
rho_hh   = params(2);
delta    = params(3);
gamma    = params(4);
alpha    = params(5);
psi      = params(6);
sigma    = params(7);
theta    = params(8);
Deltat   = params(9);
Deltaz   = params(10);
Upsilon  = params(11);
thetaP   = params(12);
epsilon  = params(13);
phiTR    = params(14); 
pibar    = params(15);
eta_q    = params(16);
timeless = params(17);
share_d  = params(18);
rhoTR    = params(19);
phiK     = params(20);
N_period = params(22);

Zhat     =  1;
wedge    =  1;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// define model

model;

mbar   = m/((epsilon-1)/epsilon);
w      = (1-alpha)*Zhat*Q^alpha*K(-1)^alpha*L^(-alpha)*mbar;    
R      = alpha*Zhat*Q^alpha*K(-1)^(alpha-1)*L^(1-alpha)*mbar*wedge;
K      = (iota-delta)*K(-1)*Deltat+K(-1);
C      = Y - ((iota+phiK/2*(iota-delta)^2)*K(-1) + thetaP*1/2*pi^2*Y); // Aggregate resource constraint
w      = Upsilon*L^psi*C;
i      = r+pi;
%i     = (1-rhoTR*Deltat)*i(-1) + rhoTR*Deltat*(rho_hh_t+phiTR*(pi-pibar)+pibar); % Taylor rule
(C(+1)-C)/(C*Deltat)= r-rho_hh_t;
r      = (R(+1)-delta*q(+1)+(q(+1)-q)/Deltat)/q(+1);
(r-(iota-delta))*(q-1-phiK*(iota-delta)) +(q*iota-iota-phiK/2*(iota-delta)^2) = (q(+1)-q)/Deltat-phiK*(iota(+1)-iota)/Deltat ;
(r-(Y(+1)-Y)/(Y*Deltat))*pi(+1)*thetaP = epsilon_t*(m-(epsilon_t-1)/epsilon_t)+thetaP*(pi(+1)-pi)/Deltat;
Y      = Zhat*(Q*K(-1))^alpha*L^(1-alpha);
T      = (1-mbar)*Y-thetaP/2*pi^2*Y - G +(iota*q-iota-phiK/2*(iota-delta)^2)*K(-1) + (R/wedge-R)*K(-1); % Profits of retailers + price adj cost + dividends - G + profits cap good producer


Welf= 0;

Aux_Va       = Deltat*(Y-((iota+phiK/2*(iota-delta)^2)*K(-1)+thetaP*1/2*pi^2*Y)-C); //Total resource constraint

// Law of motion in case of exogenous MIT shock
Q         = exp((1-eta_q*Deltat)*log(Q(-1)));                    // Exogenous TFP
rho_hh_t  = rho_hh_t(-1)*(1-eta_q*Deltat)+eta_q*Deltat*rho_hh;   // HH discount
G         = G(-1)*(1-eta_q*Deltat)+eta_q*Deltat*0;               // Govt spending
epsilon_t = epsilon_t(-1)*(1-eta_q*Deltat)+eta_q*Deltat*epsilon; // Cost push shock

end;




////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Provide SS of model conditional on pi
steady_state_model; // provide SS as function of pi without optimal policy

%pi=0;
[L,r,m,K,w,i,Y,T,C,rho_hh_t,Q,Welf,I,q,R,iota,Aux_Va,mbar]=main_RANK_ss4(pi,0);
G=0;
epsilon_t=epsilon;

end;



////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// define planner's problem

planner_objective(ln(C)-Upsilon*((L^(1+psi))/(1+psi)));
%planner_objective(-(pi)^2);
ramsey_model(planner_discount=1/(1+rho_hh*Deltat),instruments=(pi));

%oo_ramsey_policy_steady_state_file=oo_;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Find SS of model with optimal policy
options_.debug=1;

steady(maxit=20,tolf=1e-12);
check;

check;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Calculate transitional dynamic

start = oo_.steady_state;
stop  = oo_.steady_state;


// Define here the MIT shock

%start(6) =start(6)-0.07;    % Monetary policy shock
%start(13)=start(13)*0.8;    % Time preference shock
%start(11)=start(11)*0.9;    % TFP shock
%start(19)=start(19)*0.9;    % Gov spending shock
start(20)=start(20)*0.9;    % Cost-push shock


initval;
w       =start(1);
r       =start(2);
K       =start(3);
C       =start(4); 
L       =start(5);
i       =start(6); 
Y       =start(7); 
m       =start(8);
pi      =start(9); 
T       =start(10);
Q       =start(11);
Welf    =start(12);
rho_hh_t=start(13);
Aux_Va  =start(14);
q       =start(15);
R       =start(16);
iota    =start(17);
mbar    =start(18);
G       =start(19);
epsilon_t=start(20);

end;

endval;
w       =stop(1);
r       =stop(2);
K       =stop(3);
C       =stop(4); 
L       =stop(5);
i       =stop(6); 
Y       =stop(7); 
m       =stop(8);
pi      =stop(9); 
T       =stop(10);
Q       =stop(11);
Welf    =stop(12);
rho_hh_t=stop(13);
Aux_Va  =stop(14);
q       =stop(15);
R       =stop(16);
iota    =stop(17);
mbar    =stop(18);
G       =stop(19);
epsilon_t=stop(20);
end;



options_.periods = N_period;
perfect_foresight_setup;

if timeless==1
    %by default dynare does timeless
else
    oo_.endo_simul(strmatch('MULT',M_.endo_names(:,:)),1)=0;
end


perfect_foresight_solver(maxit=10,tolx=1e-10,tolf=1e-10);


%-------------------------------------------------------------
%                         GRAPHS
%-------------------------------------------------------------

shock   = epsilon_t; %% Define here the variable that is shocked for graphs

X=nan(size(K));
D=K;
A=nan(size(K));
Z=nan(size(K));
zstar=nan(size(K));
phi=nan(size(K));
Omega=nan(size(K));


I_dev     = ((iota(1:end).*K(1:end)-delta*K(end))/(delta*K(end))*100);
Y_dev     = (Y(1:end)-Y(end))/Y(end)*100;
TFP       = (Q.*X./(1-Omega)).^alpha ;
r_dev     = (r(1:end)-r(end))*100;
w_dev     = (w(1:end)-w(end))/w(end)*100;
shock_dev = (shock(1:end)-shock(end))/shock(end)*100;



Tn       = N_period*Deltat;               % Time in years - 600 is number of grids
years    = [Deltat:Deltat:Tn];            % Vector of time in years
iniT     = 2;                             % First period 

maxy  = 30;
maxT  = maxy/Deltat;                      % Last year we want for the graphs



figure(12)
subplot(6,3,1),  hold on, plot(years(iniT:maxT),(L(iniT:maxT)-L(end))/L(end)*100,'LineWidth',2)                 , title('L')
subplot(6,3,2),  hold on, plot(years(iniT:maxT),(Y(iniT:maxT)-Y(end))/Y(end)*100,'LineWidth',2)                 , title('Y')
subplot(6,3,3),  hold on, plot(years(iniT:maxT),(pi(iniT:maxT)-pi(end)),'LineWidth',2)                          , title('pi')
subplot(6,3,4),  hold on, plot(years(iniT:maxT),(K(iniT:maxT)-K(end))/K(end)*100,'LineWidth',2)                 , title('K')
subplot(6,3,5),  hold on, plot(years(iniT:maxT),(i(iniT:maxT)-i(end)),'LineWidth',2)                            , title('i')
subplot(6,3,6),  hold on, plot(years(iniT:maxT),((C(iniT:maxT)-C(end))/C(end))*100,'LineWidth',2)               , title('C')
subplot(6,3,7),  hold on, plot(years(iniT:maxT),((m(iniT:maxT)-m(end))/m(end))*100,'LineWidth',2)               , title('m')
subplot(6,3,8),  hold on, plot(years(iniT:maxT),r_dev(iniT:maxT),'LineWidth',2)                                 , title('r')
subplot(6,3,9),  hold on, plot(years(iniT:maxT),I_dev(iniT:maxT),'LineWidth',2)                                 , title('I')
subplot(6,3,10), hold on, plot(years(iniT:maxT),((zstar(iniT:maxT)-zstar(end))/zstar(end)*100),'LineWidth',2)   , title('zstar')
subplot(6,3,11), hold on, plot(years(iniT:maxT),((q(iniT:maxT)-q(end))/q(end)*100),'LineWidth',2)               , title('q')
subplot(6,3,12), hold on, plot(years(iniT:maxT),w_dev(iniT:maxT),'LineWidth',2)                                 , title('w')
subplot(6,3,13), hold on, plot(years(iniT:maxT),((phi(iniT:maxT)-phi(end))/phi(end)*100),'LineWidth',2)         , title('phi')
subplot(6,3,14), hold on, plot(years(iniT:maxT),((A(iniT:maxT)-A(end))/A(end)*100),'LineWidth',2)               , title('A')
subplot(6,3,15), hold on, plot(years(iniT:maxT),(TFP(iniT:maxT)-TFP(end))/TFP(end)*100,'LineWidth',2)           , title('TFP')
subplot(6,3,16), hold on, plot(years(iniT:maxT),(D(iniT:maxT)-D(end))/D(end)*100,'LineWidth',2)                 , title('D')
subplot(6,3,17), hold on, plot(years(iniT:maxT),R(iniT:maxT)-R(end),'LineWidth',2)                              , title('R')
subplot(6,3,18), hold on, plot(years(iniT:maxT),shock_dev(iniT:maxT),'LineWidth',2)                            , title('Shock')




